"""
Module showing off how to create a composite object.

A composite object is one that is made up of more than one game2d object, but which can
be moved about on the screen together.

Author: Walker M. White
Date: November 30, 2020
"""
from game2d import *


### Constants ###

# The initial window width
GAME_WIDTH  = 640
# The initial window height
GAME_HEIGHT = 480

# Pixels to move the object each frame
OBJECT_STEP = 2
# The width of the image
IMAGE_WIDTH  = 100
# The height of the image
IMAGE_HEIGHT = 100
# The font size
FONT_SIZE = 64
# How far below the image to put the label
FONT_OFFSET = 40

class App(GameApp):
    """
    The basic app for moving an image with arrow keys
    
    The image moves so long as the arrow keys are held down. So we do not need to
    distinguish between presses and releases
    """
    # Attribute _comp: The composite object
    # Invariant: _comp is a Combined object
    
    def start(self):
        """
        Initialize the image
        """
        pass # Implement me
    
    def update(self,dt):
        """
        Move the image about the screen
        
        Parameter dt: The time since the last animation frame
        Precondition: dt is a float
        """
        pass # Implement me
        
    def draw(self):
        """
        Draw the image
        """
        pass # Implement me


class Combined(object):
    """
    A composite object combining a GImage and GLabel
    
    Composite objects are created like subcontrollers.  They need a custom
    initializer and custom draw method.  HOWEVER, they are not proper subcontrollers
    because they do not need an update method.  Simply control them with their
    getters and setters
    """
    # Attribute _image: The image object to display
    # Invariant: _image is a GImage object
    #
    # Attribute _label: The text labe to display
    # Invariant: _label is a GLabel object
    #
    # Attribute _x: The x-coordinate of the (averaged) center
    # Invariant: _x is a number and is always the average of _image and _label
    #
    # Attribute _y: The y-coordinate of the (averaged) center
    # Invariant: _y is a number and is always the average of _image and _label
    pass # Implement me


# Script Code
if __name__ == '__main__':
    App(width=GAME_WIDTH,height=GAME_HEIGHT).run()